MinimalArchaeology = {}

MinimalArchaeology['skill'] = {}	 -- Store information about the players skill
MinimalArchaeology['artifacts'] = {} -- Store information about current artifacts
MinimalArchaeology['barlinks'] = {1,2,3,4,5,6,7,8,9,10} -- Store information about bar links
MinimalArchaeology['ArtifactBars'] = {} -- References for the artifact bars

MinimalArchaeologyHideNext = false; -- Addon is loaded, all values are set, everything is good to go
MinimalArchaeologyIsReady = false; -- Addon is loaded, all values are set, everything is good to go

-- Saved Variables
MinimalArchaeologyOptions = {} -- Store options
MinimalArchaeologyHideOptions = {} -- Store Artifact Hide specific options
MinimalArchaeologyArtifactOptions = {} -- Store Artifact specific options

SLASH_MINIMALARCHAEOLOGY1 = "/minarch"
SlashCmdList["MINIMALARCHAEOLOGY"] = function(msg, editBox)
   	if (msg == "hide") then
		MinimalArchaeologyFrame:Hide();
		MinimalArchaeologyOptions['HideFrame'] = true;
	elseif (msg == "options") then
		InterfaceOptionsFrame_OpenToCategory(MinimalArchaeologyOptionPanel);
	else
		MinimalArchaeologyFrame:Show();
		MinimalArchaeologyOptions['HideFrame'] = false;
		MinimalArchaeologyHideNext = false;
	end
end

function MinimalArchaeology:OnLoad(self)
	-- Register Events
	MinimalArchaeologyFrame:RegisterForDrag("LeftButton")
	--MinimalArchaeologyFrame:RegisterEvent("ARTIFACT_UPDATE");
	--MinimalArchaeologyFrame:RegisterEvent("ARTIFACT_HISTORY_READY");
	MinimalArchaeologyFrame:RegisterEvent("ARTIFACT_COMPLETE");
	MinimalArchaeologyFrame:RegisterEvent("ARTIFACT_DIG_SITE_UPDATED");
	MinimalArchaeologyFrame:RegisterEvent("CURRENCY_DISPLAY_UPDATE");
	MinimalArchaeologyFrame:RegisterEvent("SKILL_LINES_CHANGED");
	
	--MinimalArchaeologyFrame:RegisterEvent("BAG_UPDATE");	-- Might need this later
	
	MinimalArchaeologyFrame:RegisterEvent("ADDON_LOADED");  -- Apply SavedVariables
	
	for i=1,10 do -- Artifact Arrays
		MinimalArchaeology['artifacts'][i] = {}; 
		MinimalArchaeology['artifacts'][i]['appliedKeystones'] = 0; -- Initialize once
		MinimalArchaeology['ArtifactBars'][i] = MinimalArchaeologyFrame["artifactBar"..i];
	end
	
	ChatFrame1:AddMessage("Minimal Archaeology Initialized!");
end

function MinimalArchaeology:OnEvent(event, ...)
	if (event == "ADDON_LOADED" and MinimalArchaeology ~= nil and MinimalArchaeologyIsReady ~= true) then
		if (MinimalArchaeologyOptions['MinimapPos'] ~= nil) then
			MinimalArchaeology:MinimapButtonReposition();
		else
			MinimalArchaeologyOptions['MinimapPos'] = 45;
		end
		
		if (MinimalArchaeologyOptions['StartHidden'] == nil) then
			MinimalArchaeologyOptions['StartHidden'] = false;
		else
			if (MinimalArchaeologyOptions['StartHidden'] == true) then
				MinimalArchaeologyOptions['HideFrame'] = true;
			end
		end
		
		if (MinimalArchaeologyOptions['HideFrame'] == nil) then
			MinimalArchaeologyOptions['HideFrame'] = false;
		else
			if (MinimalArchaeologyOptions['HideFrame'] == true) then
				MinimalArchaeologyFrame:Hide();
			end
		end
		
		if (MinimalArchaeologyOptions['HideMinimap'] == nil) then
			MinimalArchaeologyOptions['HideMinimap'] = false;
		else
			if (MinimalArchaeologyOptions['HideMinimap'] == true) then
				MinimalArchaeologyMinimapButton:Hide();
			end
		end
		
		if (MinimalArchaeologyOptions['DisableSound'] == nil) then
			MinimalArchaeologyOptions['DisableSound'] = false;
		end

		if (MinimalArchaeologyOptions['HideAfterDigsite'] == nil) then
			MinimalArchaeologyOptions['HideAfterDigsite'] = false;
		end	
		
		if (MinimalArchaeologyOptions['WaitForSolve'] == nil) then
			MinimalArchaeologyOptions['WaitForSolve'] = false;
		end	
		
		if (MinimalArchaeologyOptions['FrameScale'] == nil) then
			MinimalArchaeologyOptions['FrameScale'] = 100;
		end
		
		for i=0,10 do
			if (MinimalArchaeologyHideOptions[i] == nil) then
				MinimalArchaeologyHideOptions[i] = {}; 
				MinimalArchaeologyHideOptions[i]['Hide'] = false;
			end
			
			if (MinimalArchaeologyArtifactOptions[i] == nil) then
				MinimalArchaeologyArtifactOptions[i] = {}; 
				MinimalArchaeologyArtifactOptions[i]['AlwaysUseKeystone'] = false;
			end
		end
			
		MinimalArchaeologyFrame:SetScale(tonumber(MinimalArchaeologyOptions['FrameScale'])/100);
		MinimalArchaeologyIsReady = true; -- Addon has loaded
		
		ChatFrame1:AddMessage("Minimal Archaeology Loaded!");
		
	elseif (event == "ARTIFACT_DIG_SITE_UPDATED" and MinimalArchaeologyOptions['HideAfterDigsite'] == true) then
		MinimalArchaeologyHideNext = true;
	elseif (event == "CURRENCY_DISPLAY_UPDATE" and MinimalArchaeologyHideNext == true) then
		if (MinimalArchaeologyOptions['WaitForSolve'] == true) then
			local wait = false;
			for i=1, 10 do
				if (MinimalArchaeology['artifacts'][i]['canSolve']) then
					wait = true;
				end
			end
			
			if (wait == false) then
				MinimalArchaeologyFrame:Hide();
				MinimalArchaeologyOptions['HideFrame'] = true;
				MinimalArchaeologyHideNext = false;
			end
		else
			MinimalArchaeologyFrame:Hide();
			MinimalArchaeologyOptions['HideFrame'] = true;
			MinimalArchaeologyHideNext = false;
		end
	elseif (event == "ARTIFACT_COMPLETE" and MinimalArchaeologyHideNext == true and MinimalArchaeologyOptions['WaitForSolve'] == true) then
		MinimalArchaeologyFrame:Hide();
		MinimalArchaeologyOptions['HideFrame'] = true;
		MinimalArchaeologyHideNext = false;
	end

	MinimalArchaeology:UpdateMinArch();
end

function MinimalArchaeology:UpdateMinArchSkill()	-- Update archaeology skill information
	local _, _, arch = GetProfessions();
	if arch then
		local name, _, rank, maxRank = GetProfessionInfo(arch);
		MinimalArchaeology['skill']['name'] = name;
		MinimalArchaeology['skill']['rank'] = rank;
		MinimalArchaeology['skill']['maxRank'] = maxRank;
	else
		MinimalArchaeology['skill']['name'] = "Archaeology";
		MinimalArchaeology['skill']['rank'] = 0;
		MinimalArchaeology['skill']['maxRank'] = 0;
	end
end

function MinimalArchaeology:UpdateMinArchSkillBar(SkillBar)	-- Update a status bar to represent the skill information
	local skill = MinimalArchaeology['skill'];
	
	SkillBar:SetMinMaxValues(0, skill['maxRank']);
	SkillBar:SetValue(skill['rank']);
	SkillBar.text:SetText(skill['name'].." "..skill['rank'].."/"..skill['maxRank']);
end

function MinimalArchaeology:UpdateMinArchArtifact(ArtifactIndex)	-- Update artifact details, only keep relevant data
	local numArtifacts = GetNumArtifactsByRace(ArtifactIndex);
	local RaceName, _, _, RaceitemID = GetArchaeologyRaceInfo(ArtifactIndex);
	
	MinimalArchaeology['artifacts'][ArtifactIndex]['race'] = RaceName;
	MinimalArchaeology['artifacts'][ArtifactIndex]['raceitemid'] = RaceitemID;
	
	if (numArtifacts == 0) then	-- The Artifact hasn't been discovered
		MinimalArchaeology['artifacts'][ArtifactIndex]['numKeystones'] = 0;
		MinimalArchaeology['artifacts'][ArtifactIndex]['progress'] = 0;
		MinimalArchaeology['artifacts'][ArtifactIndex]['modifier'] = 0;
		MinimalArchaeology['artifacts'][ArtifactIndex]['total'] = 0;
		MinimalArchaeology['artifacts'][ArtifactIndex]['canSolve'] = false;
		MinimalArchaeology['artifacts'][ArtifactIndex]['canSolvePrev'] = false;
	else
		SetSelectedArtifact(ArtifactIndex);
		
		-- KeyStones
		local availablekeystones = 0;		
		if (MinimalArchaeologyArtifactOptions[ArtifactIndex]['AlwaysUseKeystone']) then
			MinimalArchaeology['artifacts'][ArtifactIndex]['appliedKeystones'] = 4;
		end		
		for i=1, MinimalArchaeology['artifacts'][ArtifactIndex]['appliedKeystones'] do
			SocketItemToArtifact();			
			if (ItemAddedToArtifact(i)) then
				availablekeystones = availablekeystones + 1;
			end
		end
		
		MinimalArchaeology['artifacts'][ArtifactIndex]['appliedKeystones'] = availablekeystones;
			
		local Name, Description, Rarity, Icon, SpellDescription, NumKeystones = GetSelectedArtifactInfo();
		local Progress, Modifier, Total = GetArtifactProgress();
		
		MinimalArchaeology['artifacts'][ArtifactIndex]['numKeystones'] = NumKeystones;
		MinimalArchaeology['artifacts'][ArtifactIndex]['progress'] = Progress;
		MinimalArchaeology['artifacts'][ArtifactIndex]['modifier'] = Modifier;
		MinimalArchaeology['artifacts'][ArtifactIndex]['total'] = Total;
		MinimalArchaeology['artifacts'][ArtifactIndex]['canSolvePrev'] = MinimalArchaeology['artifacts'][ArtifactIndex]['canSolve'];
		MinimalArchaeology['artifacts'][ArtifactIndex]['canSolve'] = CanSolveArtifact();
		MinimalArchaeology['artifacts'][ArtifactIndex]['name'] = Name;
		MinimalArchaeology['artifacts'][ArtifactIndex]['rarity'] = Rarity;
		MinimalArchaeology['artifacts'][ArtifactIndex]['description'] = Description;
		MinimalArchaeology['artifacts'][ArtifactIndex]['spelldescription'] = SpellDescription;

	end
end

function MinimalArchaeology:UpdateMinArchArtifactBar(ArtifactIndex, ArtifactBar)
	local artifact = MinimalArchaeology['artifacts'][ArtifactIndex];
	local keyStars = "";
	local RuneName, _, _, _, _, _, _, _, _, runeStoneIconPath = GetItemInfo(artifact['raceitemid']);
	
	ArtifactBar:SetMinMaxValues(0, artifact['total']);
	ArtifactBar:SetValue(min(artifact['progress']+artifact['modifier'], artifact['total']));
	
	ArtifactBar.keystone.icon:SetTexture(runeStoneIconPath);
	if (artifact['appliedKeystones'] == 0) then
		ArtifactBar.keystone.icon:SetAlpha(0.1);
		ArtifactBar.keystone.text:SetText("");
	else
		ArtifactBar.keystone.icon:SetAlpha((artifact['appliedKeystones']/artifact['numKeystones']));
		ArtifactBar.keystone.text:SetText(artifact['appliedKeystones']);
	end	
	
	if (artifact['numKeystones'] > 0 and artifact['total'] > 0) then
		ArtifactBar.keystone:Show();
		ArtifactBar.keystone.icon:Show();
		for i=1,artifact['numKeystones'] do
			keyStars = "*"..keyStars;
		end
	else
		ArtifactBar.keystone:Hide();
	end
	
	if (artifact['modifier'] > 0) then
		ArtifactBar.text:SetText(keyStars..artifact['race'].." (+"..artifact['modifier']..") "..(artifact['progress']+artifact['modifier']).."/"..artifact['total']);
	else
		ArtifactBar.text:SetText(keyStars..artifact['race'].." "..artifact['progress'].."/"..artifact['total']);
	end
	
	if (artifact['canSolve']) then
		if (MinimalArchaeologyOptions['DisableSound'] == false and artifact['canSolvePrev'] ~= artifact['canSolve']) then
			PlaySoundFile("Sound\\interface\\MapPing.wav")
			artifact['canSolvePrev'] = artifact['canSolve'];
		end
		ArtifactBar.solveButton:Enable();
	else
		ArtifactBar.solveButton:Disable();
	end	
end

function MinimalArchaeology:MinArchSolve(BarIndex)
	SetSelectedArtifact(MinimalArchaeology['barlinks'][BarIndex]);
	
	for i=1, MinimalArchaeology['artifacts'][MinimalArchaeology['barlinks'][BarIndex]]['appliedKeystones'] do
		SocketItemToArtifact();
	end	
	MinimalArchaeology['artifacts'][MinimalArchaeology['barlinks'][BarIndex]]['appliedKeystones'] = 0;
	
	SolveArtifact();
end

function MinimalArchaeology:UpdateMinArch()
	-- If the addon is ready then there should be no nil values
	if (MinimalArchaeologyIsReady == true) then
		MinimalArchaeology:UpdateMinArchSkill();
		MinimalArchaeology:UpdateMinArchSkillBar(MinimalArchaeologyFrameSkillBar);
		
		local ActiveBarIndex = 0;
		
		for i=1,10 do
			MinimalArchaeology:UpdateMinArchArtifact(i); -- Update each artifact
			
			local artifact = MinimalArchaeology['artifacts'][i];
			local options = MinimalArchaeologyHideOptions[i];
			if (artifact['total'] > 0 and options['Hide'] == false) then
				ActiveBarIndex = ActiveBarIndex + 1;
				MinimalArchaeology:UpdateMinArchArtifactBar(i,MinimalArchaeology['ArtifactBars'][ActiveBarIndex]);
				MinimalArchaeology['ArtifactBars'][ActiveBarIndex]:Show();
				MinimalArchaeology['barlinks'][ActiveBarIndex] = i;
			end
		end
		
		local MinArchFrameHeight = 310;
		
		for i=ActiveBarIndex+1, 10 do
			MinimalArchaeology['ArtifactBars'][i]:Hide();
			MinArchFrameHeight = MinArchFrameHeight - 25;
		end
		
		MinimalArchaeologyFrame:SetHeight(MinArchFrameHeight);
	end
end

function MinimalArchaeology:ArtifactToolTip(BarIndex)
	local artifact = MinimalArchaeology['artifacts'][MinimalArchaeology['barlinks'][BarIndex]];
	
	GameTooltip:SetOwner(MinimalArchaeology['ArtifactBars'][BarIndex], "ANCHOR_BOTTOMRIGHT");
	
	if (artifact['rarity'] == 1) then
		GameTooltip:AddLine(artifact['name'], 0, 0.4, 0.8, 1);
	else
		GameTooltip:AddLine(artifact['name'], GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b, 1);
	end
	
	GameTooltip:AddLine(artifact['description'], 1.0, 1.0, 1.0, 1);
	GameTooltip:AddLine(artifact['spelldescription'], NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b, 1);
	GameTooltip:Show();
end

function MinimalArchaeology:KeystoneClick(self, button, down)
	local artifactIndex = MinimalArchaeology['barlinks'][self:GetID()];
	local numofappliedkeystones = MinimalArchaeology['artifacts'][artifactIndex]['appliedKeystones'];
	local numoftotalkeystones = MinimalArchaeology['artifacts'][artifactIndex]['numKeystones'];
	
	if (button == "LeftButton") then
		if (numofappliedkeystones < numoftotalkeystones) then
			 MinimalArchaeology['artifacts'][artifactIndex]['appliedKeystones'] = numofappliedkeystones + 1;
		end
	elseif (button == "RightButton") then
		if (numofappliedkeystones > 0) then
			 MinimalArchaeology['artifacts'][artifactIndex]['appliedKeystones'] = numofappliedkeystones - 1;
		end
	end
		
	MinimalArchaeology:UpdateMinArchArtifact(artifactIndex);
	MinimalArchaeology:UpdateMinArchArtifactBar(artifactIndex,MinimalArchaeology['ArtifactBars'][self:GetID()]);
	
end
